import "Turbine.UI";
import "Turbine.UI.Lotro";
import "Narufind.FowlMap";

-- the control that displays the map of a region with its corresponding locations. each location is active or not, depending on the togglebutton
MapControl = class( Turbine.UI.Window );

function MapControl:Constructor( location, parent )
	Turbine.UI.Window.Constructor( self );
	self:SetSize(800,630);
	
	self.pointCoords = {};
	self.pointLabels = {};
	self.pointLocations = {};
	self.pointButtons = {};  
	
	self.stoneCoords = {};
	self.stoneLabels = {};
	self.stoneLocations = {};
	self.stoneButtons = {};
	
	self.sneakCoords = {};
	self.sneakSizes = {};
	self.sneakLabels = {};
	self.sneakLabelText = {};
	self.sneakLabelSizes = {};
	self.sneakAreas = {};
	self.sneakButtons = {};
	
	self.locCoords = {};
	self.locLabels = {};
	self.locLocations = {};
	self.locLabelSizes = {};
	self.locButtons = {};
	
	self.location = location;
	
	self.locked = false;
	if mapdata[location] ~= nil then
		for key,value in pairs( mapdata[location] ) do
			table.insert(self.pointCoords,key);
			table.insert(self.pointLocations,value);
		end
	end
	if mapdata[location.."S"] ~= nil then
		for key,value in pairs( mapdata[location.."S"] ) do
			table.insert(self.stoneCoords,key);
			table.insert(self.stoneLocations,value);
		end
	end
	if mapdata[location.."Sneaks"] ~= nil then
		for i=1,tableLength(mapdata[location.."Sneaks"]) do
		-- for key,value in pairs( mapdata[location.."Sneaks"] ) do
			table.insert(self.sneakCoords,mapdata[location.."Sneaks"][i]["Pos"]);
			table.insert(self.sneakSizes,mapdata[location.."Sneaks"][i]["Size"]);
			table.insert(self.sneakLabelText,mapdata[location.."Sneaks"][i]["Label"]);
			table.insert(self.sneakLabelSizes,mapdata[location.."Sneaks"][i]["LabelSize"]);
			table.insert(self.sneakAreas,mapdata[location.."Sneaks"][i]["Area"]);
		end
	end
	if mapdata[location.."L"] ~= nil then
		for key,value in pairs( mapdata[location.."L"] ) do
			table.insert(self.locCoords,key);
			table.insert(self.locLocations,value);
			table.insert(self.locLabelSizes,mapdata[location.."LLabelSize"][key]);
		end
	end

	
	self.pointNr = tableLength(self.pointCoords)
	self.stoneNr = tableLength(self.stoneCoords)
	self.sneakNr = tableLength(self.sneakCoords)
	self.locNr = tableLength(self.locCoords)
	
	self.lockIcon = Turbine.UI.Button();
	self.lockIcon:SetParent( self );
	self.lockIcon:SetSize( 20,20 );
	self.lockIcon:SetPosition( 762,0 );
	self.lockIcon:SetBackground( strings["unlock"] );
	self.lockIcon:SetBlendMode( Turbine.UI.BlendMode.AlphaBlend );
	self.lockIcon.MouseClick = function( sender,args )
		self:SetLocked(not self.locked);
	end


	for i=1,self.pointNr do
		self.pointButtons[i] = MapButton( self, self.pointLocations[i][1], self.pointLocations[i][2], self.pointCoords[i] , 0, 0);
		self.pointLabels[i] = Turbine.UI.Label();
		if (self.pointButtons[i]:GetLeft()+self.pointButtons[i]:GetWidth()+3+90)<=800 then
		  self.pointLabels[i]:SetPosition( self.pointButtons[i]:GetLeft()+self.pointButtons[i]:GetWidth()+3, self.pointButtons[i]:GetTop()+self.pointButtons[i]:GetHeight()/2 );
		else
      self.pointLabels[i]:SetPosition( self.pointButtons[i]:GetLeft()-90-3, self.pointButtons[i]:GetTop()+self.pointButtons[i]:GetHeight()/2 );
		end
		self.pointLabels[i]:SetParent( self );
		self.pointLabels[i]:SetVisible( false );
		self.pointLabels[i]:SetSize( 90,10 );
		self.pointLabels[i]:SetTextAlignment( Turbine.UI.ContentAlignment.MiddleCenter );
		self.pointLabels[i]:SetText( self.pointCoords[i] );
		self.pointLabels[i]:SetBackColor( Turbine.UI.Color(1,0.1,0.1,0.1) );
		self.pointLabels[i]:SetZOrder( self.pointButtons[i]:GetZOrder()+1 );
		self.pointButtons[i].MouseEnter = function(  )
			self.pointLabels[i]:SetVisible( true );
		end
		self.pointButtons[i].MouseLeave = function()
			self.pointLabels[i]:SetVisible( false );
		end
	end
	for i=1,self.stoneNr do
		self.stoneButtons[i] = MapButton( self, self.stoneLocations[i][1], self.stoneLocations[i][2], self.stoneCoords[i] , 1, 0);
		self.stoneLabels[i] = Turbine.UI.Label();
		if (self.stoneButtons[i]:GetLeft()+self.stoneButtons[i]:GetWidth()+3+90)<=800 then
			self.stoneLabels[i]:SetPosition( self.stoneButtons[i]:GetLeft()+self.stoneButtons[i]:GetWidth()+3, self.stoneButtons[i]:GetTop()+self.stoneButtons[i]:GetHeight()/2 );
		else
			self.stoneLabels[i]:SetPosition( self.stoneButtons[i]:GetLeft()-90-3, self.stoneButtons[i]:GetTop()+self.stoneButtons[i]:GetHeight()/2 );
		end
		self.stoneLabels[i]:SetParent( self );
		self.stoneLabels[i]:SetVisible( false );
		self.stoneLabels[i]:SetSize( 90,10 );
		self.stoneLabels[i]:SetTextAlignment( Turbine.UI.ContentAlignment.MiddleCenter );
		self.stoneLabels[i]:SetText( self.stoneCoords[i] );
		self.stoneLabels[i]:SetBackColor( Turbine.UI.Color(1,0.1,0.1,0.1) );
		self.stoneLabels[i]:SetZOrder( self.stoneButtons[i]:GetZOrder()+1 );
		self.stoneButtons[i].MouseEnter = function(  )
		  self.stoneLabels[i]:SetVisible( true );
		end
		self.stoneButtons[i].MouseLeave = function()
		  self.stoneLabels[i]:SetVisible( false );
		end
	end
	for i=1,self.sneakNr do
		self.sneakButtons[i] = MapButton( self, self.sneakSizes[i][1], self.sneakSizes[i][2], self.sneakCoords[i] , 3, self.sneakAreas[i]);
        -- self.sneakButtons[i]:SetBackColor(Turbine.UI.Color.Blue);
        self.sneakButtons[i]:SetBackColorBlendMode(Turbine.UI.BlendMode.Color);
		self.sneakLabels[i] = Turbine.UI.Label();
		self.sneakLabels[i]:SetPosition( self.sneakButtons[i]:GetLeft()+self.sneakButtons[i]:GetWidth()/2 -self.sneakLabelSizes[i][1]/2, self.sneakButtons[i]:GetTop()+self.sneakButtons[i]:GetHeight() +3);
		self.sneakLabels[i]:SetParent( self );
		self.sneakLabels[i]:SetVisible( false );
		self.sneakLabels[i]:SetSize( self.sneakLabelSizes[i][1],self.sneakLabelSizes[i][2] );
		self.sneakLabels[i]:SetTextAlignment( Turbine.UI.ContentAlignment.MiddleCenter );
		self.sneakLabels[i]:SetText( self.sneakLabelText[i] );
		self.sneakLabels[i]:SetBackColor( Turbine.UI.Color(1,0.1,0.1,0.1) );
		self.sneakLabels[i]:SetZOrder( self.sneakButtons[i]:GetZOrder()+1 );
		self.sneakButtons[i].MouseEnter = function(  )
			self.sneakLabels[i]:SetVisible( true );
			self.sneakButtons[i]:SetBackground(self.sneakAreas[i][2]);
			-- self.sneakButtons[i]:SetBackColor(Turbine.UI.Color.Purple);
		end
		self.sneakButtons[i].MouseLeave = function()
			self.sneakLabels[i]:SetVisible( false );
			self.sneakButtons[i]:SetBackground(self.sneakAreas[i][1]);
			-- self.sneakButtons[i]:SetBackColor(Turbine.UI.Color.Blue);
		end
	end
	for i=1,self.locNr do
		self.locButtons[i] = MapButton( self, self.locLocations[i][1], self.locLocations[i][2], self.locCoords[i] , 4, 0);
		self.locLabels[i] = Turbine.UI.Label();
		if (self.locButtons[i]:GetLeft()+self.locButtons[i]:GetWidth()+3+self.locLabelSizes[i])<=800 then
			self.locLabels[i]:SetPosition( self.locButtons[i]:GetLeft()+self.locButtons[i]:GetWidth()+3, self.locButtons[i]:GetTop()+self.locButtons[i]:GetHeight()/2 );
		else
			self.locLabels[i]:SetPosition( self.locButtons[i]:GetLeft()-self.locLabelSizes[i]-3, self.locButtons[i]:GetTop()+self.locButtons[i]:GetHeight()/2 );
		end
		self.locLabels[i]:SetParent( self );
		self.locLabels[i]:SetVisible( false );
		self.locLabels[i]:SetSize( self.locLabelSizes[i], 22 );
		self.locLabels[i]:SetTextAlignment( Turbine.UI.ContentAlignment.MiddleCenter );
		self.locLabels[i]:SetText( self.locCoords[i] );
		self.locLabels[i]:SetBackColor( Turbine.UI.Color(1,0.1,0.1,0.1) );
		self.locLabels[i]:SetZOrder( self.locButtons[i]:GetZOrder()+1 );
		self.locButtons[i].MouseEnter = function(  )
		  self.locLabels[i]:SetVisible( true );
		end
		self.locButtons[i].MouseLeave = function()
		  self.locLabels[i]:SetVisible( false );
		end
	end

	
	if location=="Shire" then
		self.nestButton = MapButton( self, (-67.8+80.3)/(-61.3+80.3)*800,(-23.7+23.7)/(-38.0+23.7)*600, "23.7S 67.8W" , 2);
		self.nestButton.Deactivate();
		self.nestButton:SetLocked( true );
		self.nestLabel = Turbine.UI.Label();
		self.nestLabel:SetPosition( self.nestButton:GetLeft()+self.nestButton:GetWidth()+3, self.nestButton:GetTop()+self.nestButton:GetHeight()/2 );
		self.nestLabel:SetParent( self );
		self.nestLabel:SetVisible( false );
		self.nestLabel:SetSize( 230,12 );
		self.nestLabel:SetTextAlignment( Turbine.UI.ContentAlignment.MiddleCenter );
		self.nestLabel:SetBackColor( Turbine.UI.Color(1,0.1,0.1,0.1) );
		self.nestLabel:SetZOrder( self.nestButton:GetZOrder()+1 );
		self.nestLabel:SetText( "Nest to High Moors: 23.7S 67.8W" );
		self.nestButton.MouseEnter = function(  )
		  self.nestLabel:SetVisible( true );
		end
		self.nestButton.MouseLeave = function()
		  self.nestLabel:SetVisible( false );
		end
	end
  
	if location=="Ered Luin" then
		self.nestButton = MapButton( self, (-91.9+110.0)/(-86.5+110.0)*800,(-24.9+12.4)/(-30.0+12.4)*600, "24.9S 91.9W" , 2);
		self.nestButton.Deactivate();
		self.nestButton:SetLocked( true );
		self.nestLabel = Turbine.UI.Label();
		self.nestLabel:SetPosition( self.nestButton:GetLeft()-250-3, self.nestButton:GetTop()+self.nestButton:GetHeight()/2 );
		self.nestLabel:SetParent( self );
		self.nestLabel:SetVisible( false );
		self.nestLabel:SetSize( 250,12 );
		self.nestLabel:SetTextAlignment( Turbine.UI.ContentAlignment.MiddleCenter );
		self.nestLabel:SetBackColor( Turbine.UI.Color(1,0.1,0.1,0.1) );
		self.nestLabel:SetZOrder( self.nestButton:GetZOrder()+1 );
		self.nestLabel:SetText( "Nest to Trestlebridge: 24.9S 91.9W" );
		self.nestButton.MouseEnter = function(  )
		  self.nestLabel:SetVisible( true );
		end
		self.nestButton.MouseLeave = function()
		  self.nestLabel:SetVisible( false );
		end
	end
  
	if location=="Bree-Land" then
		self.nestButton = MapButton( self, (-41.9+67.2)/(-36.4+67.2)*800,(-19.9+14.8)/(-38.0+14.8)*600, "19.9S 41.9W" , 2);
		self.nestButton.Deactivate();
		self.nestButton:SetLocked( true );
		self.nestLabel = Turbine.UI.Label();
		self.nestLabel:SetPosition( self.nestButton:GetLeft()-210-3, self.nestButton:GetTop()+self.nestButton:GetHeight()/2 );
		self.nestLabel:SetParent( self );
		self.nestLabel:SetVisible( false );
		self.nestLabel:SetSize( 210,12 );
		self.nestLabel:SetTextAlignment( Turbine.UI.ContentAlignment.MiddleCenter );
		self.nestLabel:SetBackColor( Turbine.UI.Color(1,0.1,0.1,0.1) );
		self.nestLabel:SetZOrder( self.nestButton:GetZOrder()+1 );
		self.nestLabel:SetText( "Nest to Malenhad: 19.9S 41.9W" );
		self.nestButton.MouseEnter = function(  )
		  self.nestLabel:SetVisible( true );
		end
		self.nestButton.MouseLeave = function()
		  self.nestLabel:SetVisible( false );
		end
	end
  
  
	if location=="North Downs" then
		self.nestButton = MapButton( self, (-41.9+60.4)/(-32.2+60.4)*800,(-19.9+1.3)/(-22.5+1.3)*600, "19.9S 41.9W" , 2);
		self.nestButton.Deactivate();
		self.nestButton:SetLocked( true );
		self.nestLabel = Turbine.UI.Label();
		self.nestLabel:SetPosition( self.nestButton:GetLeft()+self.nestButton:GetWidth()+3, self.nestButton:GetTop()+self.nestButton:GetHeight()/2 );
		self.nestLabel:SetParent( self );
		self.nestLabel:SetVisible( false );
		self.nestLabel:SetSize( 210,12 );
		self.nestLabel:SetTextAlignment( Turbine.UI.ContentAlignment.MiddleCenter );
		self.nestLabel:SetBackColor( Turbine.UI.Color(1,0.1,0.1,0.1) );
		self.nestLabel:SetZOrder( self.nestButton:GetZOrder()+1 );
		self.nestLabel:SetText( "Nest to Malenhad: 19.9S 41.9W" );
		self.nestButton.MouseEnter = function(  )
		  self.nestLabel:SetVisible( true );
		end
		self.nestButton.MouseLeave = function()
		  self.nestLabel:SetVisible( false );
		end
		self.nestButton2 = MapButton( self, (-53.6+60.4)/(-32.2+60.4)*800,(-16.2+1.3)/(-22.5+1.3)*600, "16.2S 53.6W" , 2);
		self.nestButton2.Deactivate();
		self.nestButton2:SetLocked( true );
		self.nestLabel2 = Turbine.UI.Label();
		self.nestLabel2:SetPosition( self.nestButton2:GetLeft()+self.nestButton2:GetWidth()+3, self.nestButton2:GetTop()+self.nestButton2:GetHeight()/2 );
		self.nestLabel2:SetParent( self );
		self.nestLabel2:SetVisible( false );
		self.nestLabel2:SetSize( 210,12 );
		self.nestLabel2:SetTextAlignment( Turbine.UI.ContentAlignment.MiddleCenter );
		self.nestLabel2:SetBackColor( Turbine.UI.Color(1,0.1,0.1,0.1) );
		self.nestLabel2:SetZOrder( self.nestButton2:GetZOrder()+1 );
		self.nestLabel2:SetText( "Nest to Duillond: 16.2S 53.6W" );
		self.nestButton2.MouseEnter = function(  )
		  self.nestLabel2:SetVisible( true );
		end
		self.nestButton2.MouseLeave = function()
		  self.nestLabel2:SetVisible( false );
		end
	end
  
	if location=="Trollshaws" then
		self.nestButton = MapButton( self, (-6.0+33.4)/(0.7+33.4)*800,(-33.1+22.8)/(-48.4+22.8)*600, "33.1S 6.0W" , 2);
		self.nestButton.Deactivate();
		self.nestButton:SetLocked( true );
		self.nestLabel = Turbine.UI.Label();
		self.nestLabel:SetPosition( self.nestButton:GetLeft()-215-3, self.nestButton:GetTop()+self.nestButton:GetHeight()/2 );
		self.nestLabel:SetParent( self );
		self.nestLabel:SetVisible( false );
		self.nestLabel:SetSize( 215,12 );
		self.nestLabel:SetTextAlignment( Turbine.UI.ContentAlignment.MiddleCenter );
		self.nestLabel:SetBackColor( Turbine.UI.Color(1,0.1,0.1,0.1) );
		self.nestLabel:SetZOrder( self.nestButton:GetZOrder()+1 );
		self.nestLabel:SetText( "Nest to Oatbarton: 33.1S 6.0W" );
		self.nestButton.MouseEnter = function(  )
		  self.nestLabel:SetVisible( true );
		end
		self.nestButton.MouseLeave = function()
		  self.nestLabel:SetVisible( false );
		end
	end
  
	if location=="Angmar" then
		self.nestButton = MapButton( self, (-34.2+42.8)/(-13.3+42.8)*800,(2.2-14.6)/(-7.4-14.6)*600, "2.2N 34.2W" , 2);
		self.nestButton.Deactivate();
		self.nestButton:SetLocked( true );
		self.nestLabel = Turbine.UI.Label();
		self.nestLabel:SetPosition( self.nestButton:GetLeft()+self.nestButton:GetWidth()+3, self.nestButton:GetTop()+self.nestButton:GetHeight()/2 );
		self.nestLabel:SetParent( self );
		self.nestLabel:SetVisible( false );
		self.nestLabel:SetSize( 210,12 );
		self.nestLabel:SetTextAlignment( Turbine.UI.ContentAlignment.MiddleCenter );
		self.nestLabel:SetBackColor( Turbine.UI.Color(1,0.1,0.1,0.1) );
		self.nestLabel:SetZOrder( self.nestButton:GetZOrder()+1 );
		self.nestLabel:SetText( "Nest to Nen Harn: 2.2N 34.2W" );
		self.nestButton.MouseEnter = function(  )
		  self.nestLabel:SetVisible( true );
		end
		self.nestButton.MouseLeave = function()
		  self.nestLabel:SetVisible( false );
		end
	end
  
	if location=="Evendim" then
		self.nestButton = MapButton( self, (-67.8+82.6)/(-52.3+82.6)*800,(-23.7+1.6)/(-24.4+1.6)*600, "23.7S 67.8W" , 2);
		self.nestButton.Deactivate();
		self.nestButton:SetLocked( true );
		self.nestLabel = Turbine.UI.Label();
		self.nestLabel:SetPosition( self.nestButton:GetLeft()+self.nestButton:GetWidth()+3, self.nestButton:GetTop()+self.nestButton:GetHeight()/2 );
		self.nestLabel:SetParent( self );
		self.nestLabel:SetVisible( false );
		self.nestLabel:SetSize( 230,12 );
		self.nestLabel:SetTextAlignment( Turbine.UI.ContentAlignment.MiddleCenter );
		self.nestLabel:SetBackColor( Turbine.UI.Color(1,0.1,0.1,0.1) );
		self.nestLabel:SetZOrder( self.nestButton:GetZOrder()+1 );
		self.nestLabel:SetText( "Nest to High Moors: 23.7S 67.8W" );
		self.nestButton.MouseEnter = function(  )
		  self.nestLabel:SetVisible( true );
		end
		self.nestButton.MouseLeave = function()
		  self.nestLabel:SetVisible( false );
		end
	end


	
    
    self.rightClick = Turbine.UI.ContextMenu();
    self.menu1 = Turbine.UI.MenuItem( strings[locale]["clearmap"] );
    self.menu1.Click = function(sender,args)
        self:SetActive( {} );
    end
    self.menu2 = Turbine.UI.MenuItem( strings[locale]["fullmap"] );
    self.menu2.Click = function(sender,args)
        self:SetInactive( {} );
    end
    self.menu3 = Turbine.UI.MenuItem( strings[locale]["lockmsg"] );
    self.menu3.Click = function(sender,args)
        self:SetLocked( not self.locked );
    end 
    self.rightClick:GetItems():Add(self.menu1);
    self.rightClick:GetItems():Add(self.menu2);
    self.rightClick:GetItems():Add(self.menu3);
    
    local menu = self.rightClick
    
    self.MouseClick = function( sender, args )
        parent:Activate();
        if (args.Button == Turbine.UI.MouseButton.Right) then
            menu:ShowMenu();
        end
    end
    
    self.markqs = Turbine.UI.Lotro.Quickslot();
    self.markqs:SetParent( self );
    self.markqs:SetSize( 12,12 );
    self.markqs:SetPosition( 740,4 );
    self.markqs:SetParent(self);
    self.markqs:SetShortcut(Turbine.UI.Lotro.Shortcut(Turbine.UI.Lotro.ShortcutType.Alias, strings[locale]["loccommand"]));
    self.markqs:SetBlendMode( Turbine.UI.BlendMode.Overlay );
    self.markqs.hider = Turbine.UI.Control();
    self.markqs.hider:SetParent( self );
    self.markqs.hider:SetSize( 13,13 );
    self.markqs.hider:SetPosition( 740,4 );
    self.markqs.hider:SetMouseVisible( false );
    self.markqs.hider:SetZOrder( self.markqs:GetZOrder()+1 );
    self.markqs.hider:SetBackground( strings["mappoint"] );
    self.markqs.hider:SetBlendMode( Turbine.UI.BlendMode.AlphaBlend );
    
    self.moveqs = Turbine.UI.Lotro.Quickslot();
    self.moveqs:SetParent( self );
    self.moveqs:SetSize( 12,12 );
    self.moveqs:SetParent( self );
    if self.pointNr > 0 then --mapdata[location] ~= nil then
		self.moveqs:SetZOrder( self.pointButtons[1]:GetZOrder()+5 );
    end
    if self.stoneNr > 0 then --mapdata[location.."S"] ~= nil then
		self.moveqs:SetZOrder( self.stoneButtons[1]:GetZOrder()+5 );
    end
    if self.sneakNr > 0 then --mapdata[location.."Sneaks"] ~= nil then
		self.moveqs:SetZOrder( self.sneakButtons[1]:GetZOrder()+5 );
    end
    if self.locNr > 0 then --mapdata[location.."L"] ~= nil then
		self.moveqs:SetZOrder( self.locButtons[1]:GetZOrder()+5 );
    end
    self.moveqs:SetShortcut(Turbine.UI.Lotro.Shortcut(Turbine.UI.Lotro.ShortcutType.Alias, strings[locale]["loccommand"]));
    self.moveqs:SetVisible( false );
    self.moveqs.hider = Turbine.UI.Control();
    self.moveqs.hider:SetParent( self );
    self.moveqs.hider:SetSize( 13,13 );
    self.moveqs.hider:SetVisible( false );
    self.moveqs.hider:SetMouseVisible( false );
    self.moveqs.hider:SetZOrder( self.moveqs:GetZOrder()+1 );
    self.moveqs.hider:SetBackground( strings["mappoint"] );
    self.moveqs.hider:SetBlendMode( Turbine.UI.BlendMode.AlphaBlend );
    
--    if chicken then
--      self.markqsC = Turbine.UI.Lotro.Quickslot();
--      self.markqsC:SetParent( self );
--      self.markqsC:SetSize( 12,12 );
--      self.markqsC:SetPosition( 720,4 );
--      self.markqsC:SetParent(self);
--      self.markqsC:SetShortcut(Turbine.UI.Lotro.Shortcut(Turbine.UI.Lotro.ShortcutType.Alias, strings[locale]["localias"]));
--      self.markqsC:SetBlendMode( Turbine.UI.BlendMode.Overlay );
--      self.markqsC.hider = Turbine.UI.Control();
--      self.markqsC.hider:SetParent( self );
--      self.markqsC.hider:SetSize( 13,13 );
--      self.markqsC.hider:SetPosition( 720,4 );
--      self.markqsC.hider:SetMouseVisible( false );
--      self.markqsC.hider:SetZOrder( self.markqsC:GetZOrder()+1 );
--      self.markqsC.hider:SetBackground( strings["mappointC"] );
--      self.markqsC.hider:SetBlendMode( Turbine.UI.BlendMode.AlphaBlend );
--      
--      self.moveqsC = {};
--    end
    
    AddCallback(Turbine.Chat, "Received", function(sender, args)
            local prefixLength = strings[locale]["locprefix"]:len();
            if args.ChatType == 4 and args.Message:sub(1,prefixLength) == strings[locale]["locprefix"] then
                local loc = ParseChatLocation(args.Message:sub(prefixLength));
                if loc ~= nil then
                    local coor = mapdata[location.."c"](loc.y, loc.x);
                    self.moveqs:SetVisible( true );
                    self.moveqs.hider:SetVisible( true );
                    self.moveqs:SetPosition( coor[1]-6, coor[2]-6 );
                    self.moveqs.hider:SetPosition( coor[1]-6, coor[2]-6 );
                end
--            elseif chicken then
--                local loc = ParseChatLocationC(args.Message);
--                if loc ~= nil then
--                    local coor = mapdata[location.."c"](loc.y, loc.x);

--                    self.moveqsC = Turbine.UI.Lotro.Quickslot();
--                    self.moveqsC:SetParent( self );
--                    self.moveqsC:SetSize( 12,12 );
--                    self.moveqsC:SetParent( self );
--                    self.moveqsC:SetZOrder( self.pointButtons[1]:GetZOrder()+5 );
--                    self.moveqsC:SetZOrder( self.stoneButtons[1]:GetZOrder()+5 );
--                    self.moveqsC:SetShortcut(Turbine.UI.Lotro.Shortcut(Turbine.UI.Lotro.ShortcutType.Alias, strings[locale]["localias"]));
--                    self.moveqsC:SetVisible( false );
--                    self.moveqsC.hider = Turbine.UI.Control();
--                    self.moveqsC.hider:SetParent( self );
--                    self.moveqsC.hider:SetSize( 13,13 );
--                    self.moveqsC.hider:SetVisible( false );
--                    self.moveqsC.hider:SetMouseVisible( false );
--                    self.moveqsC.hider:SetZOrder( self.moveqsC:GetZOrder()+1 );
--                    self.moveqsC.hider:SetBackground( strings["mappointC"] );
--                    self.moveqsC.hider:SetBlendMode( Turbine.UI.BlendMode.AlphaBlend );

--                    self.moveqsC:SetVisible( true );
--                    self.moveqsC.hider:SetVisible( true );
--                    self.moveqsC:SetPosition( coor[1]-6, coor[2]-6 );
--                    self.moveqsC.hider:SetPosition( coor[1]-6, coor[2]-6 );
--                end
            end
        end);
end


function MapControl:SetMakeQS( makeQuickSlots )
    for i=1,self.pointNr do
        self.pointButtons[i].makeQS = makeQuickSlots;
    end
    for i=1,self.stoneNr do
        self.stoneButtons[i].makeQS = makeQuickSlots;
    end
    for i=1,self.sneakNr do
        self.sneakButtons[i].makeQS = makeQuickSlots;
    end
    for i=1,self.locNr do
        self.locButtons[i].makeQS = makeQuickSlots;
    end
    if self.location=="Shire" then
        self.nestButton.makeQS = makeQuickSlots;
    end
    if self.location=="Ered Luin" then
        self.nestButton.makeQS = makeQuickSlots;
    end
    if self.location=="Bree-Land" then
        self.nestButton.makeQS = makeQuickSlots;
    end
    if self.location=="North Downs" then
        self.nestButton.makeQS = makeQuickSlots;
        self.nestButton2.makeQS = makeQuickSlots;
    end
    if self.location=="Trollshaws" then
        self.nestButton.makeQS = makeQuickSlots;
    end
    if self.location=="Angmar" then
        self.nestButton.makeQS = makeQuickSlots;
    end
    if self.location=="Evendim" then
        self.nestButton.makeQS = makeQuickSlots;
    end
end


function MapControl:AddActive( coordlist )
    if not self.locked then
        for i=1,self.pointNr do
            if tableContains( coordlist, self.pointCoords[i] ) then
                self.pointButtons[i].Activate();
            end
        end
        for i=1,self.stoneNr do
            if tableContains( coordlist, self.stoneCoords[i] ) then
                self.stoneButtons[i].Activate();
            end
        end
        -- for i=1,self.sneakNr do
            -- if tableContains( coordlist, self.sneakCoords[i] ) then
                -- self.sneakButtons[i].Activate();
            -- end
        -- end
        for i=1,self.locNr do
            if tableContains( coordlist, self.locCoords[i] ) then
                self.locButtons[i].Activate();
            end
        end
    end
end

function MapControl:SetActive( coordlist )
    if not self.locked then
        for i=1,self.pointNr do
            if tableContains( coordlist, self.pointCoords[i] ) then
                self.pointButtons[i].Activate();
            else
                self.pointButtons[i].Deactivate();
            end
        end
        for i=1,self.stoneNr do
            if tableContains( coordlist, self.stoneCoords[i] ) then
                self.stoneButtons[i].Activate();
            else
                self.stoneButtons[i].Deactivate();
            end
        end
        -- for i=1,self.sneakNr do
            -- if tableContains( coordlist, self.sneakCoords[i] ) then
                -- self.sneakButtons[i].Activate();
            -- else
                -- self.sneakButtons[i].Deactivate();
            -- end
        -- end
        for i=1,self.locNr do
            if tableContains( coordlist, self.locCoords[i] ) then
                self.locButtons[i].Activate();
            else
                self.locButtons[i].Deactivate();
            end
        end
    end
end

function MapControl:SetInactive( coordlist )
    if not self.locked then
        for i=1,self.pointNr do
            if tableContains( coordlist, self.pointCoords[i] ) then
                self.pointButtons[i].Deactivate();
            else
                self.pointButtons[i].Activate();
            end
        end
        for i=1,self.stoneNr do
            if tableContains( coordlist, self.stoneCoords[i] ) then
                self.stoneButtons[i].Deactivate();
            else
                self.stoneButtons[i].Activate();
            end
        end
        -- for i=1,self.sneakNr do
            -- if tableContains( coordlist, self.sneakCoords[i] ) then
                -- self.sneakButtons[i].Deactivate();
            -- else
                -- self.sneakButtons[i].Activate();
            -- end
        -- end
        for i=1,self.locNr do
            if tableContains( coordlist, self.locCoords[i] ) then
                self.locButtons[i].Deactivate();
            else
                self.locButtons[i].Activate();
            end
        end
    end
end

function MapControl:SetDefault( coordlist )
    if not self.locked then
        for i=1,self.pointNr do
            if tableContains( coordlist, self.pointCoords[i] ) then
                self.pointButtons[i].Deactivate();
            else
                self.pointButtons[i].Activate();
            end
        end
        for i=1,self.stoneNr do
            if tableContains( coordlist, self.stoneCoords[i] ) then
                self.stoneButtons[i].Deactivate();
            else
                self.stoneButtons[i].Activate();
            end
        end
        -- for i=1,self.sneakNr do
            -- if tableContains( coordlist, self.sneakCoords[i] ) then
                -- self.sneakButtons[i].Activate();
            -- else
                -- self.sneakButtons[i].Deactivate();
            -- end
        -- end
        for i=1,self.locNr do
            if tableContains( coordlist, self.locCoords[i] ) then
                self.locButtons[i].Deactivate();
            else
                self.locButtons[i].Activate();
            end
        end
    end
end

function MapControl:SetLocked( bool )
    self.locked = bool;
    if bool then
        self.menu3:SetText( strings[locale]["unlockmsg"] );
        self.lockIcon:SetBackground( strings["lock"] );
    else
        self.menu3:SetText( strings[locale]["lockmsg"] );
        self.lockIcon:SetBackground( strings["unlock"] );
    end
    for i=1,self.pointNr do
        self.pointButtons[i]:SetLocked( bool );
    end
    for i=1,self.stoneNr do
        self.stoneButtons[i]:SetLocked( bool );
    end
    -- for i=1,self.sneakNr do
        -- self.sneakButtons[i]:SetLocked( bool );
    -- end
    for i=1,self.locNr do
        self.locButtons[i]:SetLocked( bool );
    end
end

function MapControl:GetActive( )
    local t = {}
    for i=1,self.pointNr do
        if self.pointButtons[i].isactive then
            table.insert(t, self.pointCoords[i])
        end
    end
    for i=1,self.stoneNr do
        if self.stoneButtons[i].isactive then
            table.insert(t, self.stoneCoords[i])
        end
    end
    -- for i=1,self.sneakNr do
        -- if self.sneakButtons[i].isactive then
            -- table.insert(t, self.sneakCoords[i])
        -- end
    -- end
    for i=1,self.locNr do
        if self.locButtons[i].isactive then
            table.insert(t, self.locCoords[i])
        end
    end
--    for i=1,self.nestNr do
--        if self.nestButtons[i].isactive then
--            table.insert(t, self.nestCoords[i])
--        end
--    end
    return t
end




local LOC_MESSAGES = { 
    'r(%d+) lx(%d+%.?%d*) ly(%d+%.?%d*) i%d+ ox(%d+%.?%d*) oy(%d+%.?%d*) oz(%d+%.?%d*) h(%d+%.?%d*)',
    'r(%d+) lx(%d+%.?%d*) ly(%d+%.?%d*) i%d+ cInside ox(.-%d+%.?%d*) oy(.-%d+%.?%d*) oz(.-%d+%.?%d*) h(%d+%.?%d*)',
    'r(%d+) lx(%d+%.?%d*) ly(%d+%.?%d*) i%d+ ox(%d+%.?%d*) oy(%d+%.?%d*) oz(%d+%.?%d*)',
    'r(%d+) lx(%d+%.?%d*) ly(%d+%.?%d*) i%d+ cInside ox(.-%d+%.?%d*) oy(.-%d+%.?%d*) oz(.-%d+%.?%d*)',
    -- not likely to match last --
    'r(%d+) lx(%d+%.?%d*) ly(%d+%.?%d*) ox(.-%d+%.?%d*) oy(.-%d+%.?%d*) oz(.-%d+%.?%d*) h(%d+%.?%d*)',
    'r(%d+) lx(%d+%.?%d*) ly(%d+%.?%d*) cInside ox(.-%d+%.?%d*) oy(.-%d+%.?%d*) oz(.-%d+%.?%d*) h(%d+%.?%d*)',
    'r(%d+) lx(%d+%.?%d*) ly(%d+%.?%d*) ox(.-%d+%.?%d*) oy(.-%d+%.?%d*) oz(.-%d+%.?%d*)',
    'r(%d+) lx(%d+%.?%d*) ly(%d+%.?%d*) cInside ox(.-%d+%.?%d*) oy(.-%d+%.?%d*) oz(.-%d+%.?%d*)'
};
local LOC_MESSAGESC = { 
    'r(%d+) lx(%d+%.?%d*) ly(%d+%.?%d*) i%d+ ox(%d+%.?%d*) oy(%d+%.?%d*) oz(%d+%.?%d*) h(%d+%.?%d*)',
    'r(%d+) lx(%d+%.?%d*) ly(%d+%.?%d*) i%d+ cInside ox(.-%d+%.?%d*) oy(.-%d+%.?%d*) oz(.-%d+%.?%d*) h(%d+%.?%d*)',
    'r(%d+) lx(%d+%.?%d*) ly(%d+%.?%d*) i%d+ ox(%d+%.?%d*) oy(%d+%.?%d*) oz(%d+%.?%d*)',
    'r(%d+) lx(%d+%.?%d*) ly(%d+%.?%d*) i%d+ cInside ox(.-%d+%.?%d*) oy(.-%d+%.?%d*) oz(.-%d+%.?%d*)',
    -- not likely to match last --
    'r(%d+) lx(%d+%.?%d*) ly(%d+%.?%d*) ox(.-%d+%.?%d*) oy(.-%d+%.?%d*) oz(.-%d+%.?%d*) h(%d+%.?%d*)',
    'r(%d+) lx(%d+%.?%d*) ly(%d+%.?%d*) cInside ox(.-%d+%.?%d*) oy(.-%d+%.?%d*) oz(.-%d+%.?%d*) h(%d+%.?%d*)',
    'r(%d+) lx(%d+%.?%d*) ly(%d+%.?%d*) ox(.-%d+%.?%d*) oy(.-%d+%.?%d*) oz(.-%d+%.?%d*)',
    'r(%d+) lx(%d+%.?%d*) ly(%d+%.?%d*) cInside ox(.-%d+%.?%d*) oy(.-%d+%.?%d*) oz(.-%d+%.?%d*)',
--    ': (%d+%.?%d*)N, (%d+%.?%d*)E',
--    ': (%d+%.?%d*)N, (%d+%.?%d*)W',
--    ': (%d+%.?%d*)S, (%d+%.?%d*)E',
--    ': (%d+%.?%d*)S, (%d+%.?%d*)W'
};

function ParseChatLocation( message )
    for i, suffix in ipairs(LOC_MESSAGES) do
        local r, lx, ly, ox, oy, oz, h = message:match(suffix);
        if r ~= nil then
            local x = (( math.floor( lx / 8 ) * 160 + tonum(ox) ) - 29360) / 200;
            local y = (( math.floor( ly / 8 ) * 160 + tonum(oy) ) - 24880) / 200;
            return { x = round(x,3), y = round(y,3) };
        end
    end
end

-- function ParseChatLocationC( message )
    -- for i, suffix in ipairs(LOC_MESSAGESC) do
        -- local r, lx, ly, ox, oy, oz, h = message:match(suffix);
        -- if r ~= nil then
            -- local x = (( math.floor( lx / 8 ) * 160 + tonum(ox) ) - 29360) / 200;
            -- local y = (( math.floor( ly / 8 ) * 160 + tonum(oy) ) - 24880) / 200;
            -- return { x = round(x,3), y = round(y,3) };
        -- end
    -- end
-- end

function round(num, idp)
    if idp and idp>0 then
        local mult = 10^idp
        return math.floor(num * mult + 0.5) / mult
    end
    return math.floor(num + 0.5)
end


function tonum(val) 
    if locale ~= 'en' then
        return tonumber((string.gsub(val,"%.",",")))
    else
        return tonumber(val);
    end
end
