import "Turbine";
import "Turbine.UI";
import "Turbine.UI.Lotro";
import "Turbine.Gameplay";
import "Narufind.FowlMap";

local function mapBg( location, parent )
	local bg = Turbine.UI.Control();
	bg:SetParent( parent );
	bg:SetSize( 800, 600 );
	bg:SetPosition( 0, 20+20+20 );--20 per tab row
	bg:SetBackground( strings[location] );
	bg:SetStretchMode( 2 );
	bg:SetVisible( false );
	return bg;
end

local function routeBg( location, parent)
	local route = Turbine.UI.Control();
	route:SetParent( parent );
	route:SetSize( strings[location.."RouteW"], strings[location.."RouteH"] );
	route:SetPosition( strings[location.."RouteX"], strings[location.."RouteY"]+20+20+20 );--20 per tab row
	route:SetBackground( strings[location.."Route"] );
	route:SetStretchMode( 2 );
	route:SetVisible( false );
	return route;
end

-- the main plugin window
RTRunner = class( Turbine.UI.Window );

function RTRunner:Constructor()
	Turbine.UI.Window.Constructor( self );

	local rtrunner = self;
	self.loaded=false;
	self.Update=function()
		if not self.loaded then
			self.loaded=true;
			Plugins["FowlMap"].Unload = function(self,sender,args)
				UnloadMe( rtrunner );
			end
			self:SetWantsUpdates(false);
		end
	end
	self:SetWantsUpdates(true);
	
	--Get player name
	player = Turbine.Gameplay.LocalPlayer.GetInstance();
	pname = player:GetName();
	chicken = (pname:sub(1,1)=="~");
	
	
	-- Window Dragging functions
	is_dragging = false;
	self.startDrag = function(sender, args)
		if ( args.Button ~= Turbine.UI.MouseButton.Left ) then return; end
		startX = args.X;
		startY = args.Y;
		is_dragging = true;
	end
	winHasMoved = false;
	self.endDrag = function(sender, args)
		if ( is_dragging ) then
			self:SetLeft(self:GetLeft() + (args.X - startX));
			self:SetTop(self:GetTop() + (args.Y - startY));
			is_dragging = false;
			if self:GetLeft() < -200 then
				self:SetLeft(-200);
			elseif self:GetLeft() + 600 > Turbine.UI.Display:GetWidth() then
				self:SetLeft(Turbine.UI.Display:GetWidth()-600); -- -self:GetWidth());
			end
			if self:GetTop() < 0 then
				self:SetTop(0);
			elseif self:GetTop() + 450 > Turbine.UI.Display:GetHeight() then
				self:SetTop(Turbine.UI.Display:GetHeight()-450); -- -self:GetHeight());
			end
			winHasMoved = false;
		end
	end
	self.doDrag = function(sender, args)
		if ( is_dragging ) then
			self:SetLeft(self:GetLeft() + (args.X - startX));
			self:SetTop(self:GetTop() + (args.Y - startY));
			winHasMoved = true;
		end
	end
	self.attachDragListener = function(target)
		target.MouseDown = self.startDrag;
		target.MouseUp = self.endDrag; --60
		target.MouseMove = self.doDrag;
	end
	
	-- Toggle visibility on F12 and Esc
	hudVisible = true;
	self.windowvisible = true;
	self.KeyDown=function(sender, args)
		if ( args.Action == Turbine.UI.Lotro.Action.Escape ) then
	    		self:SetVisible( false );
	  	elseif ( args.Action == 268435635 ) then
	    		hudVisible=not hudVisible;
	    		if hudVisible then
	      			self:SetVisible(self.windowvisible);
	    		else
				self.windowvisible =self:IsVisible();
				self:SetVisible(false);
			end
		end
	end
	self:SetWantsKeyEvents(true);
	

	self:SetText( "Fowl Map" );
	self:SetSize( 800,750+20+20 );--Add 20 for each addition tab row.
	self.nrMaps = 30;
	self.activeMap = 1;
	self.tabHeight=20;
	self.nrTabRows=3;
	self.nrTabRow = {};
	self.nrTabRow[1] = 9;
	self.tabRow2Start = self.nrTabRow[1]+1;
	self.nrTabRow[2] = 14;
	self.tabRow3Start = self.nrTabRow[1]+self.nrTabRow[2]+1;
	self.nrTabRow[3] = 7;
		
	self.bottomBar = Turbine.UI.Control();
	self.bottomBar:SetParent( self );
	
	self.mapsBg = {};
	--Crosser of Roads
	self.mapsBg[1] = mapBg( "Shire", self );
	self.mapsBg[2] = mapBg( "Ered Luin", self );
	self.mapsBg[3] = mapBg( "Bree-Land", self );
	self.mapsBg[4] = mapBg( "Lone-Lands", self );
	self.mapsBg[5] = mapBg( "North Downs", self );
	self.mapsBg[6] = mapBg( "Trollshaws", self );
	self.mapsBg[7] = mapBg( "Misty Mountains", self );
	self.mapsBg[8] = mapBg( "Angmar", self );
	self.mapsBg[9] = mapBg( "Evendim", self );
	--The Great Wide Open
	self.mapsBg[10] = mapBg( "Shire", self );
	self.mapsBg[11] = mapBg( "Bree-Land", self );
	self.mapsBg[12] = mapBg( "Lone-Lands", self );
	self.mapsBg[13] = mapBg( "Trollshaws", self );
	self.mapsBg[14] = mapBg( "Eregion", self );
	self.mapsBg[15] = mapBg( "Enedwaith", self );
	self.mapsBg[16] = mapBg( "Dunland", self );
	self.mapsBg[17] = mapBg( "West Rohan", self );
	self.mapsBg[18] = mapBg( "Western Gondor", self );
	self.mapsBg[19] = mapBg( "Central Gondor", self );
	self.mapsBg[20] = mapBg( "Eastern Gondor", self );
	self.mapsBg[21] = mapBg( "Far Anorien", self );
	self.mapsBg[22] = mapBg( "Old Anorien", self );
	self.mapsBg[23] = mapBg( "Minas Tirith", self );
	--There and Bawk Again
	self.mapsBg[24] = mapBg( "Shire", self );
	self.mapsBg[25] = mapBg( "Bree-Land", self );
	self.mapsBg[26] = mapBg( "Lone-Lands", self );
	self.mapsBg[27] = mapBg( "Trollshaws", self );
	self.mapsBg[28] = mapBg( "Misty Mountains", self );
	self.mapsBg[29] = mapBg( "Vales of Anduin", self );
	self.mapsBg[30] = mapBg( "Northern Mirkwood", self );
	
	self.maps = {};
	self.maps[1] = MapControl("Shire", self );
	self.maps[2] = MapControl("Ered Luin", self );
	self.maps[3] = MapControl("Bree-Land", self );
	self.maps[4] = MapControl("Lone-Lands", self );
	self.maps[5] = MapControl("North Downs", self );
	self.maps[6] = MapControl("Trollshaws", self );
	self.maps[7] = MapControl("Misty Mountains", self );
	self.maps[8] = MapControl("Angmar", self );
	self.maps[9] = MapControl("Evendim", self );
	--The Great Wide Open
	self.maps[10] = MapControl( "Shire2", self );
	self.maps[11] = MapControl( "Bree-Land2", self );
	self.maps[12] = MapControl( "Lone-Lands2", self );
	self.maps[13] = MapControl( "Trollshaws2", self );
	self.maps[14] = MapControl( "Eregion", self );
	self.maps[15] = MapControl( "Enedwaith", self );
	self.maps[16] = MapControl( "Dunland", self );
	self.maps[17] = MapControl( "West Rohan", self );
	self.maps[18] = MapControl( "Western Gondor", self );
	self.maps[19] = MapControl( "Central Gondor", self );
	self.maps[20] = MapControl( "Eastern Gondor", self );
	self.maps[21] = MapControl( "Far Anorien", self );
	self.maps[22] = MapControl( "Old Anorien", self );
	self.maps[23] = MapControl( "Minas Tirith", self );
	--There and Bawk Again
	self.maps[24] = MapControl( "Shire3", self );
	self.maps[25] = MapControl( "Bree-Land2", self );
	self.maps[26] = MapControl( "Lone-Lands2", self );
	self.maps[27] = MapControl( "Trollshaws3", self );
	self.maps[28] = MapControl( "Misty Mountains2", self );
	self.maps[29] = MapControl( "Vales of Anduin", self );
	self.maps[30] = MapControl( "Northern Mirkwood", self );
	
	self.tabs = {};
	self.tabs[1] = TabButton(strings[locale]["Shires"],strings[locale]["Shirew"],20,Turbine.UI.Color(1,0.85,0.65,0));
	self.tabs[2] = TabButton(strings[locale]["Ered Luins"],45,20,Turbine.UI.Color(1,0,0,1));
	self.tabs[3] = TabButton(strings[locale]["Bree-Lands"],45,20,Turbine.UI.Color(1,0,0.6,0.45));
	self.tabs[4] = TabButton(strings[locale]["Lone-Landss"],45,20,Turbine.UI.Color(1,0.8,0.5,0));
	self.tabs[5] = TabButton(strings[locale]["North Downss"],45,20,Turbine.UI.Color(1,0.8,0.4,0.4));
	self.tabs[6] = TabButton(strings[locale]["Trollshawss"],45,20,Turbine.UI.Color(1,0.1,0.7,0));
	self.tabs[7] = TabButton(strings[locale]["Misty Mountainss"],45,20,Turbine.UI.Color(1,0.5,0.5,0.5));
	self.tabs[8] = TabButton(strings[locale]["Angmars"],45,20,Turbine.UI.Color(1,0.8,0,0));
	self.tabs[9] = TabButton(strings[locale]["Evendims"],45,20,Turbine.UI.Color(1,0.2,0,1));
	--The Great Wide Open
	self.tabs[10] = TabButton(strings[locale]["Shire2s"],45,20,Turbine.UI.Color(1,0.3,0.4,0.5));
	self.tabs[11] = TabButton(strings[locale]["Bree-Land2s"],45,20,Turbine.UI.Color(1,0,0.5,0.65));
	self.tabs[12] = TabButton(strings[locale]["Lone-Lands2s"],45,20,Turbine.UI.Color(1,0.8,0.5,0.5));
	self.tabs[13] = TabButton(strings[locale]["Trollshaws2s"],45,20,Turbine.UI.Color(1,0.3,0.6,0.2));
	self.tabs[14] = TabButton(strings[locale]["Eregions"],45,20,Turbine.UI.Color(1,0.8,0.2,0.1));
	self.tabs[15] = TabButton(strings[locale]["Enedwaiths"],45,20,Turbine.UI.Color(1,0.45,0.35,0.7));
	self.tabs[16] = TabButton(strings[locale]["Dunlands"],45,20,Turbine.UI.Color(1,0.8,0.3,0.55));
	self.tabs[17] = TabButton(strings[locale]["West Rohans"],45,20,Turbine.UI.Color(1,0.7,0.8,0.1));
	self.tabs[18] = TabButton(strings[locale]["Western Gondors"],45,20,Turbine.UI.Color(1,0.2,0.8,0));
	self.tabs[19] = TabButton(strings[locale]["Central Gondors"],45,20,Turbine.UI.Color(1,0.4,0,1));
	self.tabs[20] = TabButton(strings[locale]["Eastern Gondors"],45,20,Turbine.UI.Color(1,0,0.6,0.45));
	self.tabs[21] = TabButton(strings[locale]["Far Anoriens"],45,20,Turbine.UI.Color(1,0.1,0.1,1));
	self.tabs[22] = TabButton(strings[locale]["Old Anoriens"],45,20,Turbine.UI.Color(1,0.2,0.55,0.45));
	self.tabs[23] = TabButton(strings[locale]["Minas Tiriths"],45,20,Turbine.UI.Color(1,0.3,0.4,0.5));
	--There and Bawk Again
	self.tabs[24] = TabButton(strings[locale]["Shire3s"],45,20,Turbine.UI.Color(1,0.26,0.75,0.45));
	self.tabs[25] = TabButton(strings[locale]["Bree-Land2s"],45,20,Turbine.UI.Color(1,0,0.8,0.35));
	self.tabs[26] = TabButton(strings[locale]["Lone-Lands2s"],45,20,Turbine.UI.Color(1,0.7,0.5,0));
	self.tabs[27] = TabButton(strings[locale]["Trollshaws3s"],45,20,Turbine.UI.Color(1,0.2,0.9,0));
	self.tabs[28] = TabButton(strings[locale]["Misty Mountains2s"],45,20,Turbine.UI.Color(1,0.5,0.5,0.5));
	self.tabs[29] = TabButton(strings[locale]["Vales of Anduins"],45,20,Turbine.UI.Color(1,0.4,0,1));
	self.tabs[30] = TabButton(strings[locale]["Northern Mirkwoods"],45,20,Turbine.UI.Color(1,0.05,0.6,0));


	self.routesBg = {};
	--The Great Wide Open
	self.routesBg[10] = routeBg( "Shire2", self );
	self.routesBg[11] = routeBg( "Bree-Land2", self );
	self.routesBg[12] = routeBg( "Lone-Lands2", self );
	self.routesBg[13] = routeBg( "Trollshaws2", self );
	self.routesBg[14] = routeBg( "Eregion", self );
	self.routesBg[15] = routeBg( "Enedwaith", self );
	self.routesBg[16] = routeBg( "Dunland", self );
	self.routesBg[17] = routeBg( "West Rohan", self );
	self.routesBg[18] = routeBg( "Western Gondor", self );
	self.routesBg[19] = routeBg( "Central Gondor", self );
	self.routesBg[20] = routeBg( "Eastern Gondor", self );
	self.routesBg[21] = routeBg( "Far Anorien", self );
	self.routesBg[22] = routeBg( "Old Anorien", self );
	self.routesBg[23] = routeBg( "Minas Tirith", self );
	--There and Bawk Again
	self.routesBg[24] = self.routesBg[10]; --Shire
	self.routesBg[25] = self.routesBg[11]; --Bree
	self.routesBg[26] = self.routesBg[12]; --Lone-Lands
	self.routesBg[27] = routeBg( "Trollshaws3", self );
	self.routesBg[28] = routeBg( "Misty Mountains2", self );
	self.routesBg[29] = routeBg( "Vales of Anduin", self );
	self.routesBg[30] = routeBg( "Northern Mirkwood", self );
	
	
	for i=1,self.nrMaps do
		self.tabs[i]:SetParent( self );
		self.maps[i]:SetParent( self );
		self.maps[i]:SetPosition( 0,self.tabHeight*self.nrTabRows );
		self.maps[i]:SetVisible( false );
		if i==1 then
			self.tabs[i]:SetPosition( 0,0 );
		elseif i<self.tabRow2Start then
			self.tabs[i]:SetPosition( self.tabs[i-1]:GetLeft()+self.tabs[i-1]:GetWidth(),0 );
		elseif i==self.tabRow2Start then
			self.tabs[i]:SetPosition( 0,self.tabHeight );
		elseif i<self.tabRow3Start then
			self.tabs[i]:SetPosition( self.tabs[i-1]:GetLeft()+self.tabs[i-1]:GetWidth(),self.tabHeight );
		elseif i==self.tabRow3Start then
			self.tabs[i]:SetPosition( 0,self.tabHeight*2 );
		else
			self.tabs[i]:SetPosition( self.tabs[i-1]:GetLeft()+self.tabs[i-1]:GetWidth(),self.tabHeight*2 );
		end
		self.tabs[i].MouseClick = function()
			SetActiveMap( self, i );
		end
		self.attachDragListener(self.maps[i]);
	end
	self.maps[self.activeMap]:SetVisible( true );
	self.maps[self.activeMap]:SetMakeQS( true );
	self.tabs[self.activeMap]:SetFront( true );
	self.mapsBg[self.activeMap]:SetVisible( true );
	if self.routesBg[self.activeMap] ~= nil then
		self.routesBg[self.activeMap]:SetVisible(true);
	end

	self.closeb = Turbine.UI.Control();
	self.closeb:SetParent( self.maps[self.activeMap] );
	self.closeb:SetSize( 16,16 );
	self.closeb:SetPosition( 783,0 );
	self.closeb:SetBackground( 0x41000196 );
	self.closeb.MouseEnter=function() self.closeb:SetBackground( 0x41000198 ) end
	self.closeb.MouseLeave=function() self.closeb:SetBackground( 0x41000196 ) end
	self.closeb.MouseDown=function() self.closeb:SetBackground( 0x41000197 ) end
	self.closeb.MouseUp=function() self.closeb:SetBackground( 0x41000196 ) end
	self.closeb.MouseClick=function() self:SetVisible( false ); self.windowvisible = false; end
	
	self.bottomBar:SetBackColor( Turbine.UI.Color(0.9,0,0,0) );
	self.bottomBar:SetSize( 800,30);
	self.bottomBar:SetPosition( 0,self:GetHeight()-130 );
	self.attachDragListener(self.bottomBar);

	self.resetButton = Turbine.UI.Lotro.Button();
	self.resetButton:SetText( strings[locale]["reset"] );
	self.resetButton:SetSize( 135,20 );
	self.resetButton:SetParent( self );
	self.resetButton:SetZOrder( self.bottomBar:GetZOrder()+1 );
	self.resetButton:SetPosition( 5,self:GetHeight()-125 );
	self.resetButton.MouseClick=function()
		for i=1,self.nrMaps do
			self.maps[i]:SetDefault( {} );
		end
	end
	
	self.sendLabel = Turbine.UI.Label();
	self.sendLabel:SetText( strings[locale]["sendtext"] );
	self.sendLabel:SetSize( 165,20 );
	self.sendLabel:SetTextAlignment( Turbine.UI.ContentAlignment.MiddleRight );
	self.sendLabel:SetParent( self );
	self.sendLabel:SetZOrder( self.bottomBar:GetZOrder()+1 );
	self.sendLabel:SetPosition( 145,self:GetHeight()-125 );
	
	self.channelList = DropDownList();
	self.channelList:SetParent( self.maps[self.activeMap] );
	if locale=="fr" then -- slight movement in case of french, since some labels are bigger
		moveleft = 12
	else
		moveleft = 0
	end
	self.channelList:SetSize( 100-moveleft,20 );
	self.channelList:SetZOrder( self.bottomBar:GetZOrder()+1 );
	self.channelList:SetPosition( 319,self:GetHeight()-185 );
	self.channelList:SetDropRows(6);
	
	
	if chicken then
		self.channelTable = strings[locale]["channelListC"];
		for _,value in pairs( self.channelTable ) do
			self.channelList:AddItem(value[1], value[2]);
		end
	else
		self.channelTable = strings[locale]["channelList"];
		for _,value in pairs( self.channelTable ) do
			self.channelList:AddItem(value[1], value[2]);
		end
	end

	self.target = Turbine.UI.Lotro.TextBox();
	self.target:SetSize( 150-moveleft,29 );
	self.target:SetParent( self );
	self.target:SetTextAlignment( Turbine.UI.ContentAlignment.MiddleLeft );
	self.target:SetZOrder( self.bottomBar:GetZOrder()+1 );
	self.target:SetPosition( 425-moveleft,self:GetHeight()-129 );
	
	self.sendButton = AliasButton( 60,20 );
	self.sendButton:SetText( strings[locale]["send"] );
	self.sendButton:SetParent( self );
	self.sendButton:SetZOrder( self.bottomBar:GetZOrder()+1 );
	self.sendButton:SetPosition( 580-2*moveleft,self:GetHeight()-125 );
	AddCallback(self.sendButton.qs, "MouseEnter", function()
		if self.channelList:GetSelectedIndex()==5 then
			self.sendButton:SetShortcutData( self.channelList:GetValue()..self.target:GetText().." "
			..self.maps[self.activeMap].location..": "
			..table.concat(self.maps[self.activeMap]:GetActive(), ", ") );
		else
			self.sendButton:SetShortcutData( self.channelList:GetValue()..self.maps[self.activeMap].location.."Locs: "
			..table.concat(self.maps[self.activeMap]:GetActive(), ", ") );
		end
	end);
	
	self.recLabel = Turbine.UI.Label();
	self.recLabel:SetText( strings[locale]["recievetext"] );
	self.recLabel:SetSize( 120+2*moveleft,20 );
	self.recLabel:SetTextAlignment( Turbine.UI.ContentAlignment.MiddleRight );
	self.recLabel:SetParent( self );
	self.recLabel:SetZOrder( self.bottomBar:GetZOrder()+1 );
	self.recLabel:SetPosition( 645-2*moveleft,self:GetHeight()-125 );
	
	self.recCheck = Turbine.UI.Lotro.CheckBox();
	self.recCheck:SetParent( self );
	self.recCheck:SetSize( 18,18 );
	self.recCheck:SetZOrder( self.bottomBar:GetZOrder()+1 );
	self.recCheck:SetPosition( 776,self:GetHeight()-124 );
	
	LoadData( self );
end

function SetActiveMap( rtrun, i )
	if i>=1 and i<=rtrun.nrMaps then
		rtrun.maps[rtrun.activeMap]:SetVisible( false );
		rtrun.maps[rtrun.activeMap]:SetMakeQS( false );
		rtrun.tabs[rtrun.activeMap]:SetFront( false );
		rtrun.tabs[rtrun.activeMap]:SetWidth( 45 );
		rtrun.tabs[rtrun.activeMap]:SetText( strings[locale][rtrun.maps[rtrun.activeMap].location.."s"] );
		rtrun.mapsBg[rtrun.activeMap]:SetVisible( false );
		if rtrun.routesBg[rtrun.activeMap] ~= nil then
			rtrun.routesBg[rtrun.activeMap]:SetVisible(false);
		end
		rtrun.activeMap = i;
		rtrun.maps[rtrun.activeMap]:SetVisible( true );
		rtrun.maps[rtrun.activeMap]:SetMakeQS( true );
		rtrun.tabs[rtrun.activeMap]:SetFront( true );
		rtrun.tabs[rtrun.activeMap]:SetWidth( strings[locale][rtrun.maps[rtrun.activeMap].location.."w"] );
		rtrun.tabs[rtrun.activeMap]:SetText( strings[locale][rtrun.maps[rtrun.activeMap].location] );
		rtrun.mapsBg[rtrun.activeMap]:SetVisible( true );
		if rtrun.routesBg[rtrun.activeMap] ~= nil then
			rtrun.routesBg[rtrun.activeMap]:SetVisible(true);
		end
		rtrun.closeb:SetParent( rtrun.maps[rtrun.activeMap] );
		rtrun.channelList:SetParent( rtrun.maps[rtrun.activeMap] );
--    for j=2,rtrun.nrMaps do
--      rtrun.tabs[j]:SetPosition( rtrun.tabs[j-1]:GetLeft()+rtrun.tabs[j-1]:GetWidth(),0 );
--    end
		for j=2,rtrun.nrMaps do
			if j<rtrun.tabRow2Start then
				rtrun.tabs[j]:SetPosition( rtrun.tabs[j-1]:GetLeft()+rtrun.tabs[j-1]:GetWidth(),0 );
			elseif j==rtrun.tabRow2Start then
				rtrun.tabs[j]:SetPosition( 0,rtrun.tabHeight );
			elseif j<rtrun.tabRow3Start then
				rtrun.tabs[j]:SetPosition( rtrun.tabs[j-1]:GetLeft()+rtrun.tabs[j-1]:GetWidth(),rtrun.tabHeight );
			elseif j==rtrun.tabRow3Start then
				rtrun.tabs[j]:SetPosition( 0,rtrun.tabHeight*2 );
			else
				rtrun.tabs[j]:SetPosition( rtrun.tabs[j-1]:GetLeft()+rtrun.tabs[j-1]:GetWidth(),rtrun.tabHeight*2 );
			end
		end
	end
end

function SaveData( rtrun )	
	local t = {}
	for i=1,rtrun.nrMaps do
		t[rtrun.maps[i].location] = rtrun.maps[i]:GetActive( );
	end
	PatchDataSave(Turbine.DataScope.Server , "FowlMcoords", t);
end

function LoadData( rtrun )
	local t = Turbine.PluginData.Load( Turbine.DataScope.Server , "FowlMcoords");
	if t~= nil then	
		for i=1,rtrun.nrMaps do
			if t[rtrun.maps[i].location]~= nil then
				rtrun.maps[i]:SetActive(t[rtrun.maps[i].location]);
			end
		end
	end
end